#
#++
#
#  Copyright Digital Equipment Corporation 1996, 1997. All Rights Reserved.
#
#  Restricted Rights: Use, duplication, or disclosure by the U.S. Government
#  is subject to restrictions as set forth in subparagraph (c)(1)(ii) of DFARS
#  252.227-7013, or in FAR 52.227-19, or in FAR 52.227-14 Alt. III, as
#  applicable.
#
#  This software is proprietary to and embodies the confidential technology of
#  Digital Equipment Corporation. Possession, use, or copying of this software
#  and media is authorized only pursuant to a valid written license from
#  Digital or an authorized sublicensor.
#
#--
#
#++
#
#  MODULE:
#
#      D_IGRP.DDL
#
#  ABSTRACT:
#
#	DDL program to Cisco IGRP packets.  The packet formats are taken
#	from Ciso's Web page at 'An Introduction to IGRP'
#
#		http://www.cisco.com/warp/public/103/5.html
#
#  AUTHORS:
#
#      A A Hart
#
#  DATE:
#
#      23 Jun 1997
#
#  MODIFICATION HISTORY:
#
#
INCLUDE ddlinc.ddl

# 6. Details of the IP Implementation
# 
# This section gives a brief description of the packet formats used by cisco's IGRP. IGRP is sent using IP
# datagrams with IP protocol 9 (IGP). The packet begins with a header. It starts immediately after the IP
# header. 
# 
#     unsigned version: 4;  /* protocol version number */
#     unsigned opcode: 4;   /* opcode */
#     uchar edition;        /* edition number */
#     ushort asystem;       /* autonomous system number */
#     ushort ninterior;     /* number of subnets in local net */
#     ushort nsystem;       /* number of networks in AS */
#     ushort nexterior;     /* number of networks outside AS */
#     ushort checksum;      /* checksum of IGRP header and data */

CONSTANT
	IGRP_version		= 0,
	IGRP_opcode		= 0,
	IGRP_edition		= 1,
	IGRP_asystem		= 2,
	IGRP_ninterior		= 4,
	IGRP_nsystem		= 6,
	IGRP_nexterior		= 8,
	IGRP_checksum		= 10,
	IGRP_sizeof		= 12;

# Here is the structure of a routing entry: 
# 
#     uchar number[3];       /* 3 significant octets of IP address */
#     uchar delay[3];        /* delay, in tens of microseconds */
#     uchar bandwidth[3];    /* bandwidth, in units of 1 Kbit/sec */
#     uchar mtu[2];          /* MTU, in octets */
#     uchar reliability;     /* percent packets successfully tx/rx */
#     uchar load;            /* percent of channel occupied */
#     uchar hopcount;        /* hop count */
# 
# The fields defined uchar[2] and uchar[3] are simply 16 and 24 bit binary integers, in normal IP network
# order. 

CONSTANT
	IGRP_number		= 0,
	IGRP_delay		= 3,
	IGRP_bandwidth		= 6,
	IGRP_mtu		= 9,
	IGRP_reliability	= 11,
	IGRP_load		= 12,
	IGRP_hopcount		= 13,
	IGRP_rte_sizeof		= 14;

DECLARE PST pst_ip_protocols;
DECLARE PST pst_igrp_rte;
DECLARE DECODE d_igrp_sub_rte, d_igrp_int_rte, d_igrp_ext_rte;


# DECODE: d_igrp_next_section
# Increment the section counter and jump to the pst_igrp_rte to jump
# jump to the next decode section.
#
# Input:
# 	CO = IGRP header.
#	$13 = section counter
#
# Output:
#	$13 = next section
#
DECODE d_igrp_next_section(0) {
	PLEVEL = HL_IGRP,
	NEXT = pst_igrp_rte
	} = 
		$13 = $13 + 1
		;



# PST: pst_igrp_subs
# Loop to displaying the contents of the local subnets section of an IGRP
# update message.
# Inputs:
#	$10 - number of local subnets
#
PST pst_igrp_subs( $10 ) {
	DEFAULT = d_igrp_sub_rte
	} =
		d_igrp_next_section
		;



# DECODE: d_igrp_sub_rte
# Decode an IGRP subnet RTE element.
# Inputs:
#	$10 - number of subnets
# Outputs:
#	$10 - decremented
#
#
DECODE d_igrp_sub_rte(0) {
	NEXT = pst_igrp_subs,
	PLEVEL = HL_IGRP,
	CONSUME = IGRP_rte_sizeof
	} =

		nl(0),
		ip_address( b3yte(IGRP_number), DONT_TRANSLATE, "Subnet" ),
		decimal( b3yte( IGRP_delay ), 0, "Delay (10ms)" ),
		decimal( b3yte( IGRP_bandwidth ), 0, "Bandwidth (1Kb/s)" ),
		decimal( word( IGRP_mtu ), 0, "MTU size" ),
		decimal( byte( IGRP_reliability ), 0, "Reliability" ),
		decimal( byte( IGRP_load ), 0, "Load" ),
		decimal( byte( IGRP_hopcount ), 0, "Hop count" ),
		$10 = $10 - 1
		;



# PST: pst_igrp_ints
# Loop to displaying the contents of the interior nets section of an IGRP
# update message.
# Inputs:
#	$11 - number of interior networks
#
PST pst_igrp_ints( $11 ) {
	DEFAULT = d_igrp_int_rte
	} =
		d_igrp_next_section
		;



# DECODE: d_igrp_int_rte
# Decode an IGRP interior RTE element.
# Inputs:
#	$11 - number of interior networks
# Outputs:
#	$11 - decremented
#
DECODE d_igrp_int_rte(0) {
	NEXT = pst_igrp_ints,
	PLEVEL = HL_IGRP,
	CONSUME = IGRP_rte_sizeof
	} =

		nl(0),
		$1 = b3yte( IGRP_number ),
		$1 = $1 << 8,
		ip_address( $1, DONT_TRANSLATE, "Interior Network" ),
		decimal( b3yte( IGRP_delay ), 0, "Delay (10ms)" ),
		decimal( b3yte( IGRP_bandwidth ), 0, "Bandwidth (1Kb/s)" ),
		decimal( word( IGRP_mtu ), 0, "MTU size" ),
		decimal( byte( IGRP_reliability ), 0, "Reliability" ),
		decimal( byte( IGRP_load ), 0, "Load" ),
		decimal( byte( IGRP_hopcount ), 0, "Hop count" ),
		$11 = $11 - 1
		;


# PST: pst_igrp_exts
# Loop to displaying the contents of the interior nets section of an IGRP
# update message.
# Inputs:
#	$12 - number of exterior networks
#
PST pst_igrp_exts( $12 ) {
	DEFAULT = d_igrp_ext_rte
	} =
		d_igrp_next_section
		;


# DECODE: d_igrp_ext_rte
# Decode an IGRP exterior RTE element.
# Inputs:
#	$12 - number of exterior networks
# Outputs:
#	$12 - decremented
#
DECODE d_igrp_ext_rte(0) {
	NEXT = pst_igrp_exts,
	PLEVEL = HL_IGRP,
	CONSUME = IGRP_rte_sizeof
	} =

		nl(0),
		$1 = b3yte( IGRP_number ),
		$1 = $1 << 8,
		ip_address( $1, DONT_TRANSLATE, "Exterior Network" ),
		decimal( b3yte( IGRP_delay ), 0, "Delay (10ms)" ),
		decimal( b3yte( IGRP_bandwidth ), 0, "Bandwidth (1Kb/s)" ),
		decimal( word( IGRP_mtu ), 0, "MTU size" ),
		decimal( byte( IGRP_reliability ), 0, "Reliability" ),
		decimal( byte( IGRP_load ), 0, "Load" ),
		decimal( byte( IGRP_hopcount ), 0, "Hop count" ),
		$12 = $12 - 1
		;



# PST: pst_igrp_rte
# Switch on the section counter in $13 and jump to the appropriate
# section decode.
#
# Input:
#	$13 = section counter
#
PST pst_igrp_rte( $13 ) =
		0 -> pst_igrp_subs,
		1 -> pst_igrp_ints,
		2 -> pst_igrp_exts
		;


# DECODE: d_igrp_update
# Input:
# 	CO = IGRP header.
#	$13 = section counter
#
DECODE d_igrp_update(1) {
	TITLE = "Update",
	NEXT = pst_igrp_rte,
	PLEVEL = HL_IGRP,
	CONSUME = IGRP_sizeof
	} = 
		$13 = 0
		;


# DECODE: d_igrp_request
# Input:
# 	CO = IGRP header.
#
DECODE d_igrp_request(2) {
	TITLE = "Request",
	PLEVEL = HL_IGRP,
	CONSUME = IGRP_sizeof
	} = 
		;


# PST: pst_igrp
# Switches on the IGRP opcode
# Co = IGRP packet.
#
PST pst_igrp( bitfield_byte( 4, 4, IGRP_opcode) ) {
	TITLE = "IGRP",
	TYPE = "IGRP"
	} = 
		d_igrp_update,
		d_igrp_request
		;


# DECODE: d_igrp_header
# Decode the IGRP header.
# Input:
# 	CO = IGRP header
# Output:
#	$10 = Number of subnets in local net
#	$11 = Number of nets inside AS
#	$12 = Number of nets outside AS
#
DECODE d_igrp_header(0) {
	TITLE = "IGRP",
	PLEVEL = HL_IGRP,
	NEXT = pst_igrp
	} =

		print( "Version", "V%d", bitfield_byte( 4, 0, IGRP_version) ),
		decimal( bitfield_byte( 4, 4, IGRP_opcode), 0, "Opcode" ),
		decimal( byte( IGRP_edition) , 0, "Edition" ),
		decimal( word( IGRP_asystem) , 0, "Autonomous system" ),
		$10 = word( IGRP_ninterior),			# Local subnets
		$11 = word( IGRP_nsystem),			# Networks inside AS
		$12 = word( IGRP_nexterior),			# Networks outside AS
		decimal( $10, 0, "Subnets in local net" ),
		decimal( $11, 0, "Nets inside AS" ),
		decimal( $12, 0, "Nets outside AS" ),
		$1 = (IGRP_rte_sizeof * ($10 + $11 + $12)) + IGRP_sizeof,
		length( $1, 0, "" ),
		$1 = ($1 << 16) + IGRP_checksum,
		ip_checksum( IGRP_checksum, $1, "Verified checksum" )
		;


# DEOCODE: d_igrp
# Main decode for Cisco's IGRP, attaches to pst_ip_protocols.
# CO = IP V4 header
# $32 = IP header length
#
DECODE d_igrp(9) {
	TITLE = "IGRP",
	BRIEF_TITLE = "",
	ATTACH = pst_ip_protocols,
	CONSUME = $32,
	NEXT = d_igrp_header,
	PLEVEL = HL_IGRP
	} =
		;
